function [ idx ] = FELRSC( X, L, kernel, var1, var2 )
% -------------FELRSC algorithm--------------------
% FAST CLUSTERING IN LINEAR INDEPENDENT 1D SUBSPACES
% The algorithm finds a dictionary that represents data in reproducible kernel Hilbert space (RKHS).
% Afterwards, data are projected into RKHS by using empirical kernel map (EKM). 
% Segmentation into subspaces is realized by applying the max operator on projected data.
%---------------------------------------------------
%
% INPUT
%   X           : is the data where each column is a given sample.
%   L           : number of subspaces
%
%   optional parameters
%   kernfunc    : kernel function 'polynomial' or 'gaussian'
%   var1,var2   : parameters for kernels
%
% Return: 
%   idx     : vector of cluster indices for each data point
%-------------------------------------------------------
% Algorithm steps:
% 1. Select randomly square root of data points if number of datapoints is large. Otherwise, use all data points. 
% Run the FVS method on selected points to find dictionary A.
% 2. Perform EKM-based mapping 
% 3. Apply the max criterion to recover subspace indexes.
%--------------------------------------------------------------------------------------------
% Ivica Kopriva, Maria Brbić, Dijana Tolić, Nino Antulov-Fantulin.
% FAST CLUSTERING IN LINEAR INDEPENDENT 1D SUBSPACES: SEGMENTATION of MULTI-CHANNEL IMAGES WITH HIGH SPATIAL RESOLUTION
% Institue Rudjer Boskovic, Zagreb, Croatia, 2016
% e-mail: ivica.kopriva@irb.hr
%--------------------------------------------------------------------------------------------
%

% Fill in unset optional values.
switch nargin
    case 2
        kernel = 'gaussian';   % 'polynomial'; 'gaussian'; 'exponential';

        var1 = 1e-1;           %  kernel parameters
                       %  for Gaussian, exponential and laplacian kernel: variance (sigma2 = var1)
        var2 = 1e-1;              % for poly kernel: offset parameter (b = var2)  
end

X = X/max(max(abs(X))); %scale X

                                    
switch(kernel)

            case 'polynomial'
                % polynomial kernel with degree equal to kparam
                kernfunc = @(X,Y)polykernel(X,Y,var1,var2);

            case 'gaussian'
                % gaussian kernel with sigma equal to kparam
                kernfunc = @(X,Y)gausskernel(X,Y,var2);
 end
 
 rN = L; % maximal number of support vectors
 
 if (size(X,2) >= 10^6)
     Ndp = 5*ceil(sqrt(size(X,2)));
     randX = datasample(X,Ndp,2); %samples are random columns from X
     fprintf('***Running FVS ...\n');
     dataFVS = FVSmod( kernfunc, randX', rN );
     ind = dataFVS.S;
     V  = randX(:,ind);
 else
     
     FVSlen = size(X,2);      % whole dataset is used
     fprintf('***Running FVS ...\n');
     dataFVS = FVSmod( kernfunc, X', rN );
     ind = dataFVS.S;
     V  = X(:,ind);
 end
 
 disp(V);
 
 % use EKM to project input image on a subspace in RKHS
 X_ekm = empirical_kernel_map(X,V,kernel,0,var1,var2);
 
 %max operator
[mv, idx] = max(X_ekm);



